package ru.ulstu.extractor.markup.service;

import org.springframework.stereotype.Service;
import ru.ulstu.extractor.markup.model.MarkupForm;
import ru.ulstu.extractor.markup.model.TimeSeriesForMarkup;
import ru.ulstu.extractor.rule.model.AntecedentValue;
import ru.ulstu.extractor.rule.service.AntecedentValueService;
import ru.ulstu.extractor.rule.service.DbRuleService;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class MarkupService {
    private final DbRuleService dbRuleService;
    private final AntecedentValueService antecedentValueService;

    public MarkupService(DbRuleService dbRuleService,
                         AntecedentValueService antecedentValueService) {
        this.dbRuleService = dbRuleService;
        this.antecedentValueService = antecedentValueService;
    }

    @Transactional
    public void generateRules(MarkupForm markupForm) {
        markupForm.getTimeSeriesForMarkupList().forEach(markupRow -> {
            Set<Map.Entry<TimeSeries, AntecedentValue>> timeSeriesEntry1 = markupRow.getTimeSeriesTendencyMap().entrySet();
            Set<Map.Entry<TimeSeries, AntecedentValue>> timeSeriesEntry2 = markupRow.getTimeSeriesTendencyMap().entrySet();
            for (Map.Entry<TimeSeries, AntecedentValue> entry1 : timeSeriesEntry1) {
                for (Map.Entry<TimeSeries, AntecedentValue> entry2 : timeSeriesEntry2) {
                    if (!entry1.equals(entry2)) {
                        dbRuleService.saveRule(entry1.getKey().getTimeSeriesType(), entry1.getValue(),
                                entry2.getKey().getTimeSeriesType(), entry2.getValue(),
                                markupRow.getMarkup());
                    }
                }
            }
        });
    }

    private AntecedentValue getAntecedent(Double diff) {
        String antecedentValue;
        if (diff < 0) {
            antecedentValue = "спад";
        } else if (diff > 0) {
            antecedentValue = "рост";
        } else {
            antecedentValue = "стабильно";
        }
        return antecedentValueService.getByValue(antecedentValue);
    }


    public List<TimeSeriesForMarkup> getTimeSeriesForMarkup(List<TimeSeries> timeSeriesList) {
        List<TimeSeriesForMarkup> tsForMarkup = new ArrayList<>();
        for (int i = 0; i < timeSeriesList.size(); i++) {
            TimeSeries ts1 = timeSeriesList.get(i);
            for (int j = i + 1; j < timeSeriesList.size(); j++) {
                TimeSeries ts2 = timeSeriesList.get(j);
                List<TimeSeriesValue> values1 = ts1.getValues();
                for (int k = 0; k < values1.size() - 1; k++) {
                    List<TimeSeriesValue> values2 = ts2.getValues();
                    for (int l = 0; l < values2.size() - 1; l++) {
                        if (values1.get(k).getDate().equals(values2.get(l).getDate())
                                && values1.get(k + 1).getDate().equals(values2.get(l + 1).getDate())) {
                            final int index = k;
                            TimeSeriesForMarkup found = tsForMarkup
                                    .stream()
                                    .filter(m -> m.getDateFrom().equals(values1.get(index).getDate()) && m.getDateTo().equals(values1.get(index + 1).getDate()))
                                    .findAny()
                                    .orElse(null);
                            AntecedentValue antecedentValue1 = getAntecedent(values1.get(index + 1).getValue() - values1.get(index).getValue());
                            AntecedentValue antecedentValue2 = getAntecedent(values2.get(l + 1).getValue() - values2.get(l).getValue());
                            if (found == null) {
                                tsForMarkup.add(new TimeSeriesForMarkup(ts1, antecedentValue1, ts2, antecedentValue2, values1.get(index).getDate(), values1.get(index + 1).getDate()));
                            } else {
                                found.getTimeSeriesTendencyMap().put(ts1, antecedentValue1);
                                found.getTimeSeriesTendencyMap().put(ts2, antecedentValue2);
                            }
                        }
                    }
                }
            }
        }
        return tsForMarkup
                .stream()
                .sorted(Comparator.comparing(TimeSeriesForMarkup::getDateFrom))
                .collect(Collectors.toList());
    }
}
